/* Main table */
table#sodar-tl-table {
  display: inline-table;
}
table#sodar-tl-table tbody tr td:nth-child(1){
  white-space: nowrap;
}
table#sodar-tl-table tbody tr td:nth-child(2) {
  width: 100%;
}

/* Detail table */
table#sodar-tl-table-detail tbody tr td:nth-child(1),
table#sodar-tl-table-detail tbody tr td:nth-child(3) {
  white-space: nowrap;
}
table#sodar-tl-table-detail tbody tr td:nth-child(2) {
  width: 100%;
}
table#sodar-tl-table-extra-data tbody tr td:nth-child(1) {
  white-space: nowrap;
  width: 100%;
}

a.sodar-tl-link-detail:hover {
  text-decoration: underline;
}
a.sodar-tl-link-extra-data:hover {
  text-decoration: underline;
  cursor: pointer;
}
a.sodar-tl-link-extra {
  cursor: pointer;
}
a.sodar-tl-link-status-extra-data:hover {
  cursor: pointer;
}
.popover {
  font-size: 100%; /* Fix for Bootstrap 4.4 */
}

/* Responsive modifications */
@media screen and (max-width: 1000px) {
  .table#sodar-tl-table tbody tr td:nth-child(1) {
    white-space: normal;
  }
}
@media screen and (max-width: 700px) {
  .table#sodar-tl-table thead tr th:nth-child(3),
  .table#sodar-tl-table tbody tr td:nth-child(3) {
    display: none;
  }
}

.sodar-tl-modal-dialog {
  max-width: 75% !important;
  font-family: monospace;
}
pre.sodar-tl-json {
  font-family: monospace;
  font-size: 85%;
}
.json-open-bracket, .json-close-bracket {
  color: #CC0011;
}
.json-property {
  color: #1A5988;
}
.json-semi-colon {
  color: #14171A;
}
.json-value {
  color: #222222;
}

.sodar-tl-modal-tabs {
  flex: 1;
  flex-direction: row;
  display: flex;
  flex-wrap: wrap;
  border-bottom: 1px solid #dee2e6;
}
.sodar-tl-copy-btn {
  position: absolute;
  top: 10px;
  right: 10px;
}
