body {
  /* HACK for non-working CSS override in django-pagedown */
  font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto,
               "Helvetica Neue", Arial, "Noto Sans", sans-serif,
               "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol",
               "Noto Color Emoji";
}

blockquote {
  border-left: 2px dotted #888;
  padding-left: 5px;
  background: #d0f0ff;
}

.wmd-panel {
  margin-left: 0;
  margin-right: 0;
  width: 100%;
  resize: vertical;
}

.wmd-button-bar  {
  width: 100%;
  background-color: #fff;
}

.wmd-input,
.sodar-json-input {
  height: 175px !important;
  min-height: 175px;
  width: 100%;
  background-color: Gainsboro;
  border: 1px solid DarkGray;
  resize: vertical;
}

.wmd-preview {
  padding: 12px;
  background-color: #c0e0ff;
}

.wmd-button-row {
  position: relative;
  margin: 10px 5px 5px;
  padding: 0;
  height: 20px;
}

.wmd-spacer {
  width: 1px;
  height: 20px;
  margin-left: 14px;
  position: absolute;
  background-color: Silver;
  display: inline-block;
  list-style: none;
}

.wmd-button {
  width: 20px;
  height: 20px;
  padding-left: 2px;
  padding-right: 3px;
  position: absolute;
  display: inline-block;
  list-style: none;
  cursor: pointer;
}

.wmd-button > span {
  background-image: url(../images/wmd-buttons.png);
  background-repeat: no-repeat;
  background-position: 0px 0px;
  width: 20px;
  height: 20px;
  display: inline-block;
}

.wmd-spacer1 {
  left: 50px;
}
.wmd-spacer2 {
  left: 175px;
}
.wmd-spacer3 {
  left: 300px;
}

.wmd-prompt-background {
  background-color: Black;
}

.wmd-prompt-dialog {
  border: 1px solid #999999;
  background-color: #F5F5F5;
}

.wmd-prompt-dialog > div {
  font-size: 0.8em;
  font-family: arial, helvetica, sans-serif;
}

.wmd-prompt-dialog > form > input[type="text"] {
  border: 1px solid #999999;
  color: black;
}

.wmd-prompt-dialog > form > input[type="button"] {
  border: 1px solid #888888;
  font-family: trebuchet MS, helvetica, sans-serif;
  font-size: 0.8em;
  font-weight: bold;
}

.wmd-preview table,
.wmd-preview table thead th,
.wmd-preview table tbody td {
  border: 1px solid #000000;
  padding-left: 5px;
  padding-right: 5px;
}
