{% load rules %}
{% load projectroles_common_tags %}

<button class="btn btn-secondary dropdown-toggle sodar-list-dropdown
               sodar-pr-role-btn-owner"
        type="button" data-toggle="dropdown"
        aria-haspopup="true" aria-expanded="false">
  <i class="iconify" data-icon="mdi:gear"></i>
</button>
<div class="dropdown-menu dropdown-menu-right sodar-pr-role-dropdown-owner">
  <a class="dropdown-item sodar-pr-role-item-transfer
            {% if not enable_owner_transfer %}disabled{% endif %}"
     href="{% url 'projectroles:role_owner_transfer' project=project.sodar_uuid %}">
    <i class="iconify" data-icon="mdi:account-switch"></i> Transfer Ownership
  </a>
  {% get_history_dropdown obj=user project=project as history_item %}
  {{ history_item|safe }}
</div>
