{% extends 'projectroles/base.html' %}

{% load rules %}

{% block title %}Confirm Admin Alert Deletion{% endblock %}

{% block projectroles %}

<div class="container-fluid sodar-subtitle-container">
  <h2>Confirm Admin Alert Deletion</h2>
</div>

<div class="container-fluid sodar-page-container">
  <div class="alert alert-warning" role="alert">
    Do you really want to delete the following admin alert?<br />
    "{{ object.message }}"
  </div>
  <form method="post">
    {% csrf_token %}
    <div class="btn-group pull-right" role="group">
      <a role="button" class="btn btn-secondary"
         href="{{ request.session.real_referer }}">
        <i class="iconify" data-icon="mdi:arrow-left-circle"></i> Cancel
      </a>
      <button type="submit" class="btn btn-danger">
        <i class="iconify" data-icon="mdi:close-thick"></i> Delete
      </button>
    </div>
  </form>
</div>

{% endblock projectroles %}
