# Generated by Django 4.2.18 on 2025-02-04 12:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0001_squashed_0032_alter_appsetting_value"),
    ]

    operations = [
        migrations.AlterField(
            model_name="sodaruseradditionalemail",
            name="secret",
            field=models.CharField(
                help_text="Secret token for email verification",
                max_length=255,
                null=True,
                unique=True,
            ),
        ),
    ]
