{% load siteinfo_tags %}

{% if pl_data.settings %}
  <div class="card" id="sodar-si-core-settings-card-app-{{ plugin.name }}">
    <div class="card-header">
      <h4>
        <i class="iconify" data-icon="{{ plugin.icon }}"></i>
          {{ plugin.title }} Settings
      </h4>
    </div>
    <div class="card-body">
      <dl class="row">
        {% for k, v in pl_data.settings.items %}
          <dt class="col-sm-5 text-nowrap">{{ k }}</dt>
          {% if v.set %}
            <dd class="{% get_info_cls v.value 'col-lg-7' %}">{% get_info_val v.value %}</dd>
          {% else %}
            <dd class="col-lg-7 text-muted">(Unset)</dd>
          {% endif %}
        {% endfor %}
      </dl>
    </div>
  </div>
{% endif %}