{% load rules %}
{% load projectroles_tags %}
{% load projectroles_common_tags %}
{% load timeline_tags %}


<tr class="sodar-pr-project-list-item" id="sodar-pr-project-search-item-{{ event.sodar_uuid }}">
  <td>
    {% get_timestamp event as event_time %}
    {{ event_time }}
  </td>
  {% if event.description %}
    <td>
      <div class="sodar-overflow-container">
        {% get_app_icon_html event plugin_lookup as event_icon %}
        <span class="badge bg-secondary text-white mr-1 sodar-obj-badge sodar-tl-event-badge">
          {{ event_icon|safe }}
          {% get_event_name event %}
        </span>
        {% if event.user %}
          {% get_user_badge event.user 'mr-1' as user_badge %}
          {{ user_badge | safe }}
        {% endif %}
        {% if event.project %}
          {% get_project_badge event.project extra_class='mr-1' as project_badge %}
          {{ project_badge | safe }}
        {% endif %}
        {% get_event_description t plugin_lookup request as event_desc %}
          {{ event_desc|safe }}
      </div>
    </td>
  {% else %}
    <td class="text-muted">N/A</td>
  {% endif %}
  <td class="{% get_status_style event.get_status %} text-light sodar-tl-item-status">
    {{ event.get_status.status_type }}
  </td>
</tr>
