<div class="card sodar-search-card">
  <div class="card-header">
    <h4>
      <i class="iconify"
         data-icon="{% if icon %}{{ icon }}{% else %}{{ plugin.icon }}{% endif %}"></i>
      {{ search_title }}
      {% if result_count %}
       ({{ result_count }})
      {% endif %}
      <div class="input-group sodar-header-input-group sodar-header-input-group-search pull-right">
        <select class="form-control sodar-search-page-length">
          <option selected value="{{ search_pagination }}">Page</option>
          <option value="{{ search_pagination }}">{{ search_pagination }} (default)</option>
          <option value="10">10</option>
          <option value="25">25</option>
          <option value="50">50</option>
          <option value="100">100</option>
          <option value="-1">All</option>
        </select>
        <input class="form-control sodar-search-filter"
           type="text" placeholder="Filter" aria-label="Filter {{ search_title }}" />
      </div>
    </h4>
  </div>
  {% if result_count >= result_limit %}
    <div class="card-body sodar-card-body-info">
      <i class="iconify" data-icon="mdi:alert"></i>
      Some results may be omitted, please narrow down your search.
    </div>
  {% endif %}
  <div class="card-body sodar-search-card-body">
