{% extends 'projectroles/project_base.html' %}

{% load rules %}
{% load projectroles_tags %}
{% load projectroles_common_tags %}

{% block title %}
  Confirm Deletion of {% get_display_name object.type True %}
{% endblock title %}

{% block projectroles_extend %}

<div class="row sodar-subtitle-container">
  <h3>Confirm Deletion of {% get_display_name object.type True %}</h3>
</div>
<div class="container-fluid sodar-page-container">
  <div class="alert alert-danger" role="alert">
    Are you sure you want to delete this {% get_display_name object.type %}?
    {% if object.is_project %}
       This will also delete all data within the project.
    {% endif %}
    This action can <strong>NOT</strong> be undone!
  </div>

  <form method="post">
    {% csrf_token %}
    <input class="form-control mb-3"
           name="delete_host_confirm"
           autocomplete="off"
           required="true"
           placeholder="Type the host name of this server (e.g. &quot;site.example.com&quot;) to confirm deletion." />
    <div class="btn-group pull-right">
      <a role="button" class="btn btn-secondary"
          href="{% url 'projectroles:update' project=object.sodar_uuid %}">
        <i class="iconify" data-icon="mdi:arrow-left-circle"></i> Cancel
      </a>
      <button type="submit" class="btn btn-danger sodar-btn-submit-once"
              id="sodar-pr-btn-confirm-delete">
        <i class="iconify" data-icon="mdi:close-thick"></i> Delete
      </button>
    </div>
  </form>
</div>

{% endblock projectroles_extend %}
