{% load bgjobs_tags %}

{% with item=item|specialize_job %}
  <tr id="sodar-bg-table-row-{{ item.sodar_uuid }}">
    <td>{{ item.bg_job.date_created|date:"Y/m/d H:i" }}</td>
    <td>{{ item.bg_job.date_modified|date:"Y/m/d H:i" }}</td>
    <td>{{ item.get_human_readable_type }}</td>
    <td>{{ item.bg_job.status }}</td>
    <td>
      {% if item.get_absolute_url %}
        <a href="{{ item.get_absolute_url }}">
          {{ item.bg_job.name }}
        </a>
      {% else %}
        {{ item.bg_job.name }}
      {% endif %}
    </td>
  </tr>
{% endwith %}
