{# Site modal dialog #}

<div class="modal" id="sodar-modal" tabindex="-1" role="dialog"
     aria-labelledby="sodar-modal-label" aria-hidden="true">
  <div class="modal-dialog modal-dialog-centered" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title" id="sodar-modal-label">Modal title</h5>
        <span class="modal-header-extra text-nowrap ml-auto"></span>
        <button type="button" class="close ml-0" data-dismiss="modal" aria-label="Close">
          <i class="iconify" data-icon="mdi:close" aria-hidden="true"></i>
        </button>
      </div>
      <div class="modal-body">
        ...
      </div>
    </div>
  </div>
</div>
<div class="modal" id="sodar-modal-wait" tabindex="-1" role="dialog" aria-hidden="true">
  <div class="modal-dialog modal-dialog-centered">
    <div class="display-3 w-100 h-100 text-center align-middle">
      <i class="iconify spin text-light" data-icon="mdi:loading"></i>
    </div>
  </div>
</div>
