# Generated by Django 4.2.22 on 2025-06-19 10:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0035_populate_viewer_role'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='public_access',
            field=models.ForeignKey(
                blank=True,
                help_text='Allow public access of specific read-only access '
                'level for project. Includes access for unauthenticated users '
                'if allowed on site.',
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='public_access',
                to='projectroles.role',
            ),
        ),
    ]
