{% extends 'projectroles/project_base.html' %}

{% load rules %}

{% block title %}
  Confirm External Link Deletion
{% endblock title %}

{% block projectroles_extend %}

<div class="container-fluidd sodar-subtitle-container">
  <h3>Confirm External Link Deletion</h3>
</div>

<div class="container-fluid sodar-page-container">
  <div class="alert alert-warning" role="alert">
    Do you really want to delete the link "{{ object.name }}"?
  </div>

  <form method="post">
    {% csrf_token %}
    <div class="row">
      <div class="btn-group ml-auto">
        <a role="button" class="btn btn-secondary"
          {% if object.folder %}
            href="{% url 'filesfolders:list' folder=object.folder.sodar_uuid %}"
          {% else %}
            href="{% url 'filesfolders:list' project=object.project.sodar_uuid %}"
          {% endif %}
          >
          <i class="iconify" data-icon="mdi:arrow-left-circle"></i> Cancel
        </a>
        <button type="submit" class="btn btn-danger">
          <i class="iconify" data-icon="mdi:close-thick"></i> Delete
        </button>
      </div>
    </div>
  </form>
</div>

{% endblock projectroles_extend %}
