{# Site footer #}
{% load projectroles_common_tags %}

{% get_django_setting 'SITE_TITLE' as site_title %}

Set the content for your footer in include/_footer.html.
{{ site_title }} v{% site_version %} / SODAR Core v{% core_version %}
