<tr class="sodar-tk-list-item">
  <td class="text-right text-muted">{{ forloop.counter }}</td>
  <td class="text-nowrap">{{ item.created|date:'Y-m-d H:i' }}</td>
  <td class="text-nowrap">{{ token.expiry|date:'Y-m-d H:i'|default:'Never' }}</td>
  <td class="text-nowrap"><code>{{ item.token_key }}</code></td>
  <td class="text-right">
    {% if request.user.is_superuser or not site_read_only %}
      {% include 'tokens/_token_dropdown.html' %}
    {% endif %}
  </td>
</tr>
