{% extends 'projectroles/base.html' %}

{% load crispy_forms_tags %}

{% block title %}
  Create API Token for {{ project.title }}
{% endblock title %}

{% block projectroles %}

<div class="container-fluid sodar-subtitle-container">
  <h2>Create API Token</h2>
</div>
<div class="container-fluid sodar-page-container">
  <form method="post">{% csrf_token %}
    {{ form|crispy }}
    <div class="text-right mb-3">
      <div class="btn-group" role="group">
        <a role="button" class="btn btn-secondary"
            href="{% url 'tokens:list' %}">
          <i class="iconify" data-icon="mdi:arrow-left-circle"></i> Cancel
        </a>
        <button type="submit" class="btn btn-primary sodar-btn-submit-once">
          <i class="iconify" data-icon="mdi:check-bold"></i> Create
        </button>
      </div>
    </div>
  </form>
</div>

{% endblock projectroles %}
