{% extends 'projectroles/project_base.html' %}

{% load rules %}

{% block title %}
  Confirm Invite Revocation
{% endblock title %}

{% block projectroles_extend %}

<div class="row sodar-subtitle-container">
  <h3>Confirm Invite Revocation</h3>
</div>

<div class="container-fluid sodar-page-container">
  <div class="alert alert-warning" role="alert">
    Are you sure you want to permanently revoke the invite sent to {{ invite.email }}?
  </div>

  <form method="post"
        action="{% url 'projectroles:invite_revoke' projectinvite=invite.sodar_uuid %}">
    {% csrf_token %}
    <div class="row">
      <div class="btn-group ml-auto">
        <a role="button" class="btn btn-secondary"
           href="{{ request.session.real_referer }}">
          <i class="iconify" data-icon="mdi:arrow-left-circle"></i> Cancel
        </a>
        <button type="submit" class="btn btn-danger">
          <i class="iconify" data-icon="mdi:close-thick"></i> Revoke
        </button>
      </div>
    </div>
  </form>
</div>

{% endblock projectroles_extend %}
