{% load bgjobs_tags %}

<table class="table table-striped sodar-card-table sodar-bg-detail-table">
  <thead>
    <tr>
      <th>Created</th>
      <th>Last Changed</th>
      <th>Type</th>
      <th id="sodar-bg-list-header-status">Status</th>
      <th id="sodar-bg-list-header-name">Name</th>
    </tr>
  </thead>
  <tbody>
    {% get_details_backgroundjobs project as backgroundjobs %}
    {% if backgroundjobs.count > 0 %}
      {% for backgroundjob in backgroundjobs %}
        {% include 'bgjobs/_backgroundjob_item.html' with item=backgroundjob details_card_mode=True %}
      {% endfor %}
    {% else %}
        <tr>
          <td class="bg-faded font-italic text-center" colspan="5">No background jobs.</td>
        </tr>
    {% endif %}
  </tbody>
</table>
