{% extends 'projectroles/base.html' %}

{% load rules %}
{% load static %}
{% load projectroles_tags %}
{% load projectroles_common_tags %}

{% block title %}
  Advanced Search
{% endblock title %}

{% block projectroles %}

<div class="row sodar-pr-content-title">
  <h2 class="sodar-pr-content-title pt-2">Advanced Search</h2>
</div>

<div class="container-fluid sodar-page-container">
  <form method="POST">
    {% csrf_token %}
    <div class="form-group">
      <label for="id_m" class="col-form-label">Search Terms</label>
      <textarea name="m" cols="40" rows="8" class=" form-control" id="id_m"></textarea>
      <small class="form-text text-muted">
        Enter one or more search terms separated by line breaks.
        Terms shorter than 3 characters will be ignored.
      </small>
    </div>
    <div class="form-group">
      <label for="id_k" class="col-form-label">Keywords</label>
      <input type="text" name="k" class="form-control"
             placeholder="key:value key2:value" id="id_k" />
      <small class="form-text text-muted">
        Enter keywords as key/value pairs separated by spaces (optional)
      </small>
    </div>
    <div class="row">
      <div class="btn-group ml-auto" role="group">
        <button type="submit" class="btn btn-primary">
          <i class="iconify" data-icon="mdi:search"></i> Search
        </button>
      </div>
    </div>
  </form>
</div>

{% endblock projectroles %}
